@@ -186,12 +186,10 @@ class IBMDB2TableStore(SQLTableStore):
         table_name, schema = super().resolve_alias(table, stage_name)
         return PipedagDB2Reflection.resolve_alias(self.engine, table_name, schema)
 
-    def check_materialization_details_supported(self, label: str | None) -> None:
-        _ = label
-        return
-
-    def _set_materialization_details(self, materialization_details: dict[str, dict[str | list[str]]] | None) -> None:
-        self.materialization_details = IBMDB2MaterializationDetails.create_materialization_details_dict(
+    def _create_materialization_details(
+        self, materialization_details: dict[str, dict[str | list[str]]] | None
+    ) -> BaseMaterializationDetails:
+        return IBMDB2MaterializationDetails.create_materialization_details_dict(
             materialization_details,
             self.strict_materialization_details,
             self.default_materialization_details,
