@@ -131,7 +131,7 @@ def materialize(
     :param ordering_barrier:
         If true, the task will be surrounded by a GroupNode(ordering_barrier=True).
         If a dictionary is provided, it is surrounded by a
-        GroupNode(**ordering_barrier). This allows passing style, style_tag, and label
+        ``GroupNode(**ordering_barrier)``. This allows passing style, style_tag, and label
         arguments.
     :param call_context:
         An optional context manager function that is opened before the task or its
@@ -346,7 +346,7 @@ def input_stage_versions(
     :param ordering_barrier:
         If true, the task will be surrounded by a GroupNode(ordering_barrier=True).
         If a dictionary is provided, it is surrounded by a
-        GroupNode(**ordering_barrier). This allows passing style, style_tag, and label
+        ``GroupNode(**ordering_barrier)``. This allows passing style, style_tag, and label
         arguments.
         Attention: In contrast to @materialize, the default value is True.
     :param call_context:
@@ -358,7 +358,7 @@ def input_stage_versions(
     :param lock_source_stages:
         If true, lock the other stage version when a ConfigContext object is passed to
         this task.
-    :param pass_args
+    :param pass_args:
         A list of named arguments that would be passed from the call to the task
         function. By default, no arguments are passed, and just tables are extracted.
 
@@ -497,7 +497,7 @@ def input_stage_versions(
         lock_source_stages,
     ):
         _task = TaskContext.get().task
-        stage = _task.stage
+        stage = _task._stage
         pass_kwargs = {k: v for k, v in kwargs.items() if k in set(pass_args)}
 
         (
@@ -600,10 +600,14 @@ def input_stage_versions(
                             other_dict[key] = cfg2.store.dematerialize_item(tbl2, input_type)
                         else:
                             other_dict[key] = (
-                                f"Table not found in other stage version: {PipedagJSONEncoder().encode(tbl2)}"
+                                f"Table/Blob not found in other stage version: {PipedagJSONEncoder().encode(tbl2)}"
                             )
                     except Exception as e:
-                        _task.logger.error(
+                        if cfg1._swallow_exceptions:
+                            log = _task._logger.info
+                        else:
+                            log = _task._logger.error
+                        log(
                             "Failed to dematerialize table from other stage version",
                             table=ref,
                             stage=tbl2.stage,
@@ -725,10 +729,10 @@ def _get_output_from_store(
     from pydiverse.pipedag.context.run_context import DematerializeRunContext
     from pydiverse.pipedag.materialize.store import dematerialize_output_from_store
 
-    root_task = task if isinstance(task, Task) else task.task
+    root_task = task if isinstance(task, Task) else task._task
 
     store = ConfigContext.get().store
-    with DematerializeRunContext(root_task.flow, allow_write_local_table_cache=write_local_table_cache):
+    with DematerializeRunContext(root_task._flow, allow_write_local_table_cache=write_local_table_cache):
         cached_output, _ = store.retrieve_most_recent_task_output_from_cache(
             root_task, ignore_position_hashes=ignore_position_hashes
         )
