@@ -2,7 +2,6 @@
 # SPDX-License-Identifier: BSD-3-Clause
 import functools
 import operator
-import types
 from dataclasses import dataclass
 
 import polars as pl
@@ -11,35 +10,22 @@ import sqlalchemy as sa
 import structlog
 from polars.testing import assert_frame_equal
 
+import pydiverse.common as pdc
 from pydiverse.pipedag import Flow, Stage, materialize, materialize_table
 from pydiverse.pipedag.context.context import CacheValidationMode, ConfigContext
 from pydiverse.pipedag.errors import HookCheckException
-from tests.fixtures.instances import DATABASE_INSTANCES, with_instances
-
-try:
-    import pydiverse.colspec as cs
-    import pydiverse.common as pdc
-    import pydiverse.transform as pdt
-except ImportError:
-    cs = types.ModuleType("pydiverse.colspec")
-    fn = lambda *args, **kwargs: (lambda *args, **kwargs: None)  # noqa: E731
-    cs.__getattr__ = lambda name: object if name in ["ColSpec", "Collection"] else fn
-    pdt = types.ModuleType("pydiverse.colspec")
-    pdt.Table = None
-    pdt.verb = lambda fn: fn
-    pdt.SqlAlchemy = None
-    pdt.Polars = None
-    pdc = None
-
-try:
-    import dataframely as dy
-except ImportError:
-    dy = None
-
+from pydiverse.pipedag.optional_dependency.adbc import adbc_postgres
+from pydiverse.pipedag.optional_dependency.colspec import cs
+from pydiverse.pipedag.optional_dependency.dataframely import dy
+from pydiverse.pipedag.optional_dependency.transform import C, pdt
+from tests.fixtures.instances import DATABASE_INSTANCES, skip_instances, with_instances
+from tests.util import swallowing_raises, tasks_library
 
 pytestmark = [
     pytest.mark.pdtransform,
-    with_instances(DATABASE_INSTANCES),
+    with_instances(tuple(list(DATABASE_INSTANCES))),
+    # snowflake currently is not supported by transform and colspec (problems could be solvable)
+    # with_instances(tuple(list(DATABASE_INSTANCES) + ["snowflake"])),
 ]
 
 # ------------------------------------------------------------------------------------ #
@@ -115,7 +101,7 @@ def data_with_filter_with_rule_violation() -> tuple[pdt.Table, pdt.Table]:
 
 
 @pytest.mark.skipif(cs.Collection is object, reason="ColSpec needs to be installed")
-@pytest.mark.skipif(pdt.Table is None, reason="pydiverse.transform needs to be installed")
+@pytest.mark.skipif(C is None, reason="pydiverse.transform needs to be installed")
 def test_dataclass():
     first, second = data_without_filter_without_rule_violation()
     c = SimpleCollection(first, second)
@@ -151,8 +137,9 @@ def exec_filter(c: cs.Collection):
 # -------------------------------------- FILTER -------------------------------------- #
 
 
+@skip_instances("postgres" if adbc_postgres is None else "")
 @pytest.mark.skipif(cs.Collection is object, reason="ColSpec needs to be installed")
-@pytest.mark.skipif(pdt.Table is None, reason="pydiverse.transform needs to be installed")
+@pytest.mark.skipif(C is None, reason="pydiverse.transform needs to be installed")
 def test_filter_without_filter_without_rule_violation():
     @materialize(input_type=pl.LazyFrame)
     def assertions(out, failure, failure_counts: dict[str, int]):
@@ -178,7 +165,7 @@ def test_filter_without_filter_without_rule_violation():
 
 
 @pytest.mark.skipif(cs.Collection is object, reason="ColSpec needs to be installed")
-@pytest.mark.skipif(pdt.Table is None, reason="pydiverse.transform needs to be installed")
+@pytest.mark.skipif(C is None, reason="pydiverse.transform needs to be installed")
 def test_filter_without_filter_with_rule_violation():
     @materialize(input_type=pl.LazyFrame)
     def assertions(out, failure, failure_counts: dict[str, int]):
@@ -200,8 +187,9 @@ def test_filter_without_filter_with_rule_violation():
     flow.run()
 
 
+@skip_instances("postgres" if adbc_postgres is None else "")
 @pytest.mark.skipif(cs.Collection is object, reason="ColSpec needs to be installed")
-@pytest.mark.skipif(pdt.Table is None, reason="pydiverse.transform needs to be installed")
+@pytest.mark.skipif(C is None, reason="pydiverse.transform needs to be installed")
 def test_filter_with_filter_without_rule_violation():
     @materialize(input_type=pl.LazyFrame)
     def assertions(out, failure, failure_counts: dict[str, int]):
@@ -235,8 +223,9 @@ def test_filter_with_filter_without_rule_violation():
     flow.run()
 
 
+@skip_instances("postgres" if adbc_postgres is None else "")
 @pytest.mark.skipif(cs.Collection is object, reason="ColSpec needs to be installed")
-@pytest.mark.skipif(pdt.Table is None, reason="pydiverse.transform needs to be installed")
+@pytest.mark.skipif(C is None, reason="pydiverse.transform needs to be installed")
 def test_filter_with_filter_with_rule_violation():
     @materialize(input_type=pl.LazyFrame)
     def assertions(out, failure, failure_counts: dict[str, int]):
@@ -271,9 +260,10 @@ def test_filter_with_filter_with_rule_violation():
 
 @pytest.mark.skipif(cs.Collection is object, reason="ColSpec needs to be installed")
 @pytest.mark.skipif(
-    dy is not None,
+    dy.Column is not None,
     reason="This test only works if dataframely is not installed since we test a fallback mechanism in polars hook",
 )
+@pytest.mark.skipif(C is None, reason="pydiverse.transform needs to be installed")
 @pytest.mark.parametrize(
     "with_filter, with_violation, validate_get_data",
     [(a, b, c) for a in [False, True] for b in [False, True] for c in [False, True]],
@@ -296,7 +286,7 @@ def test_annotations(with_filter: bool, with_violation: bool, validate_get_data:
     def consumer(first: MyFirstColSpec, second: MySecondColSpec):
         assert [(c.name, c.dtype()) for c in first] == [
             ("a", pdc.Int64()),
-            ("b", pdc.Int16() if validate_get_data or dy else pdc.Int64()),
+            ("b", pdc.Int16() if validate_get_data or dy.Column else pdc.Int64()),
             ("c", pdc.String()),
         ]
         assert [(c.name, c.dtype()) for c in second] == [
@@ -307,7 +297,7 @@ def test_annotations(with_filter: bool, with_violation: bool, validate_get_data:
         assert len(first >> pdt.export(pdt.Polars())) == 3
         assert len(second >> pdt.export(pdt.Polars())) in [3, 4, 5]
 
-        if not validate_get_data and dy is None:
+        if not validate_get_data and dy.Column is None:
             # colspec will not do casting without dataframely
             # In case of validate_get_data, the type should already be
             # smallint in the database.
@@ -322,7 +312,7 @@ def test_annotations(with_filter: bool, with_violation: bool, validate_get_data:
             with pytest.raises(cs.exc.RuleValidationError, match="2 rules failed validation"):
                 MySecondColSpec.validate(second)
         else:
-            if not validate_get_data and dy is None:
+            if not validate_get_data and dy.Column is None:
                 # colspec will not do casting without dataframely
                 # In case of validate_get_data, the type should already be
                 # smallint in the database.
@@ -334,7 +324,7 @@ def test_annotations(with_filter: bool, with_violation: bool, validate_get_data:
     def consumer2(first: MyFirstColSpec, second: MySecondColSpec):
         assert [(c.name, c.dtype()) for c in first] == [
             ("a", pdc.Int64()),
-            ("b", pdc.Int16() if validate_get_data or dy else pdc.Int64()),
+            ("b", pdc.Int16() if validate_get_data or dy.Column else pdc.Int64()),
             ("c", pdc.String()),
         ]
         assert [(c.name, c.dtype()) for c in second] == [
@@ -355,7 +345,7 @@ def test_annotations(with_filter: bool, with_violation: bool, validate_get_data:
 
     if with_violation and validate_get_data:
         # Validation at end of get_anno_data task fails
-        with pytest.raises(
+        with swallowing_raises(
             HookCheckException,
             match="failed validation with MyFirstColSpec; Failure counts: {'b|min': 1, 'c|nullability': 1};"
             if with_filter
@@ -378,9 +368,10 @@ def test_annotations(with_filter: bool, with_violation: bool, validate_get_data:
 
 @pytest.mark.skipif(cs.Collection is object, reason="ColSpec needs to be installed")
 @pytest.mark.skipif(
-    dy is not None,
+    dy.Column is not None,
     reason="This test only works if dataframely is not installed since we test a fallback mechanism in polars hook",
 )
+@pytest.mark.skipif(C is None, reason="pydiverse.transform needs to be installed")
 @pytest.mark.parametrize(
     "with_filter, with_violation, validate_get_data",
     [(a, b, c) for a in [False, True] for b in [False, True] for c in [False, True]],
@@ -403,7 +394,7 @@ def test_annotations_not_fail_fast(with_filter: bool, with_violation: bool, vali
     def consumer(first: MyFirstColSpec, second: MySecondColSpec):
         assert [(c.name, c.dtype()) for c in first] == [
             ("a", pdc.Int64()),
-            ("b", pdc.Int16() if validate_get_data or dy else pdc.Int64()),
+            ("b", pdc.Int16() if validate_get_data or dy.Column else pdc.Int64()),
             ("c", pdc.String()),
         ]
         assert [(c.name, c.dtype()) for c in second] == [
@@ -414,7 +405,7 @@ def test_annotations_not_fail_fast(with_filter: bool, with_violation: bool, vali
         assert len(first >> pdt.export(pdt.Polars())) == 3
         assert len(second >> pdt.export(pdt.Polars())) in [3, 4, 5]
 
-        if not validate_get_data and dy is None:
+        if not validate_get_data and dy.Column is None:
             # colspec will not do casting without dataframely
             # In case of validate_get_data, the type should already be
             # smallint in the database.
@@ -426,7 +417,7 @@ def test_annotations_not_fail_fast(with_filter: bool, with_violation: bool, vali
     def consumer2(first: MyFirstColSpec, second: MySecondColSpec):
         assert [(c.name, c.dtype()) for c in first] == [
             ("a", pdc.Int64()),
-            ("b", pdc.Int16() if validate_get_data or dy else pdc.Int64()),
+            ("b", pdc.Int16() if validate_get_data or dy.Column else pdc.Int64()),
             ("c", pdc.String()),
         ]
         assert [(c.name, c.dtype()) for c in second] == [
@@ -445,7 +436,7 @@ def test_annotations_not_fail_fast(with_filter: bool, with_violation: bool, vali
         with Stage("s02"):
             consumer2(first, second)
 
-    with ConfigContext.get().evolve(fail_fast=False):
+    with ConfigContext.get().evolve(fail_fast=False, swallow_exceptions=True):
         result = flow.run(cache_validation_mode=CacheValidationMode.FORCE_CACHE_INVALID)
     if with_violation:
         assert not result.successful
@@ -455,9 +446,10 @@ def test_annotations_not_fail_fast(with_filter: bool, with_violation: bool, vali
 
 @pytest.mark.skipif(cs.Collection is object, reason="ColSpec needs to be installed")
 @pytest.mark.skipif(
-    dy is not None,
+    dy.Column is not None,
     reason="This test only works if dataframely is not installed since we test a fallback mechanism in polars hook",
 )
+@pytest.mark.skipif(C is None, reason="pydiverse.transform needs to be installed")
 @pytest.mark.parametrize(
     "with_filter, with_violation, validate_get_data",
     [(a, b, c) for a in [False, True] for b in [False, True] for c in [False, True]],
@@ -482,7 +474,9 @@ def test_annotations_fault_tolerant(with_filter: bool, with_violation: bool, val
             ("a", pdc.Int64()),
             (
                 "b",
-                pdc.Int16() if (validate_get_data and (not with_violation or with_filter)) or dy else pdc.Int64(),
+                pdc.Int16()
+                if (validate_get_data and (not with_violation or with_filter)) or dy.Column
+                else pdc.Int64(),
             ),
             ("c", pdc.String()),
         ]
@@ -494,7 +488,7 @@ def test_annotations_fault_tolerant(with_filter: bool, with_violation: bool, val
         assert len(first >> pdt.export(pdt.Polars())) == 3
         assert len(second >> pdt.export(pdt.Polars())) in [3, 4, 5]
 
-        if (not validate_get_data or (with_violation and not with_filter)) and dy is None:
+        if (not validate_get_data or (with_violation and not with_filter)) and dy.Column is None:
             # colspec will not do casting without dataframely
             # In case of validate_get_data, the type should already be
             # smallint in the database.
@@ -519,7 +513,9 @@ def test_annotations_fault_tolerant(with_filter: bool, with_violation: bool, val
             ("a", pdc.Int64()),
             (
                 "b",
-                pdc.Int16() if (validate_get_data and (not with_violation or with_filter)) or dy else pdc.Int64(),
+                pdc.Int16()
+                if (validate_get_data and (not with_violation or with_filter)) or dy.Column
+                else pdc.Int64(),
             ),
             ("c", pdc.String()),
         ]
@@ -550,10 +546,7 @@ def test_annotations_fault_tolerant(with_filter: bool, with_violation: bool, val
 
 
 @pytest.mark.skipif(cs.Collection is object, reason="ColSpec needs to be installed")
-@pytest.mark.skipif(
-    dy is not None,
-    reason="This test only works if dataframely is not installed since we test a fallback mechanism in polars hook",
-)
+@pytest.mark.skipif(C is None, reason="pydiverse.transform needs to be installed")
 @pytest.mark.parametrize(
     "with_filter, with_violation, validate_get_data",
     [(a, b, c) for a in [False, True] for b in [False, True] for c in [False, True]],
@@ -630,6 +623,9 @@ def test_collections(with_filter: bool, with_violation: bool, validate_get_data:
         name = f"with{'out' if not with_filter else ''}_filter_with{'out' if not with_violation else ''}_rule_violation"
         with Stage("s01"):
             collection = get_anno_collection(name)
+            tasks_library.noop(collection.first)  # extract single schema of collection
+            tasks_library.noop(collection.second)
+            tasks_library.noop_lazy(collection.first)  # extract single schema of collection
             consumer_collection(collection)
         with Stage("s02"):
             consumer2_collection(collection)
@@ -647,9 +643,10 @@ def test_collections(with_filter: bool, with_violation: bool, validate_get_data:
 
 @pytest.mark.skipif(cs.Collection is object, reason="ColSpec needs to be installed")
 @pytest.mark.skipif(
-    dy is not None,
+    dy.Column is not None,
     reason="This test only works if dataframely is not installed since we test a fallback mechanism in polars hook",
 )
+@pytest.mark.skipif(C is None, reason="pydiverse.transform needs to be installed")
 def test_type_mapping():
     @materialize(nout=2)
     def get_anno_data() -> tuple[MyFirstColSpec, MySecondColSpec]:
@@ -677,6 +674,7 @@ def standardize_dtype(dtype):
 
 
 @pytest.mark.skipif(cs.Collection is object, reason="ColSpec needs to be installed")
+@pytest.mark.skipif(C is None, reason="pydiverse.transform needs to be installed")
 @pytest.mark.parametrize(
     "with_filter, with_violation, validate_get_data",
     [(a, b, c) for a in [False, True] for b in [False, True] for c in [False, True]],
@@ -763,7 +761,7 @@ def test_annotations_sql(with_filter: bool, with_violation: bool, validate_get_d
 
     if with_violation and validate_get_data:
         # Validation at end of get_anno_data task fails
-        with pytest.raises(
+        with swallowing_raises(
             HookCheckException,
             match="failed validation with MyFirstColSpec; Failure counts: {'b|min': 1, 'c|nullability': 1};"
             if with_filter
