@@ -1,6 +1,5 @@
 # Copyright (c) QuantCo and pydiverse contributors 2024-2025
 # SPDX-License-Identifier: BSD-3-Clause
-import types
 from dataclasses import dataclass
 
 import polars as pl
@@ -12,21 +11,14 @@ from polars.testing import assert_frame_equal
 from pydiverse.pipedag import Flow, Stage, materialize
 from pydiverse.pipedag.context.context import CacheValidationMode, ConfigContext
 from pydiverse.pipedag.errors import HookCheckException
-from tests.fixtures.instances import DATABASE_INSTANCES, with_instances
-
-try:
-    import dataframely as dy
-
-    import pydiverse.colspec as cs
-except ImportError:
-    cs = types.ModuleType("pydiverse.colspec")
-    fn = lambda *args, **kwargs: (lambda *args, **kwargs: None)  # noqa: E731
-    cs.__getattr__ = lambda name: object if name in ["ColSpec", "Collection"] else fn
-    dy = None
-
+from pydiverse.pipedag.optional_dependency.colspec import cs
+from pydiverse.pipedag.optional_dependency.dataframely import dy
+from tests.fixtures.instances import DATABASE_INSTANCES, skip_instances, with_instances
+from tests.util import swallowing_raises
 
 pytestmark = [
-    with_instances(DATABASE_INSTANCES),
+    # with_instances(tuple(list(DATABASE_INSTANCES))),
+    with_instances(tuple(list(DATABASE_INSTANCES) + ["snowflake"])),
 ]
 
 
@@ -109,7 +101,7 @@ def test_dataclass():
 
 
 @pytest.mark.skipif(cs.Collection is object, reason="ColSpec needs to be installed")
-@pytest.mark.skipif(dy is None, reason="dataframely needs to be installed")
+@pytest.mark.skipif(dy.Column is None, reason="dataframely needs to be installed")
 def test_enum_violation():
     second = pl.LazyFrame({"a": [2, 3, 4, 5], "b": [0, 1, 2, 3], "c": ["z", "y", "y", "x"]})
     # it is expected that cast fails on invalid enum value
@@ -151,7 +143,7 @@ def exec_filter_polars(c: cs.Collection):
 
 
 @pytest.mark.skipif(cs.Collection is object, reason="ColSpec needs to be installed")
-@pytest.mark.skipif(dy is None, reason="dataframely needs to be installed")
+@pytest.mark.skipif(dy.Column is None, reason="dataframely needs to be installed")
 def test_filter_without_filter_without_rule_violation():
     @materialize(input_type=pl.LazyFrame)
     def assertions(out, failure, failure_counts: dict[str, int]):
@@ -175,7 +167,7 @@ def test_filter_without_filter_without_rule_violation():
 
 
 @pytest.mark.skipif(cs.Collection is object, reason="ColSpec needs to be installed")
-@pytest.mark.skipif(dy is None, reason="dataframely needs to be installed")
+@pytest.mark.skipif(dy.Column is None, reason="dataframely needs to be installed")
 def test_filter_without_filter_with_rule_violation():
     @materialize(input_type=pl.LazyFrame)
     def assertions(out, failure, failure_counts: dict[str, int]):
@@ -198,7 +190,7 @@ def test_filter_without_filter_with_rule_violation():
 
 
 @pytest.mark.skipif(cs.Collection is object, reason="ColSpec needs to be installed")
-@pytest.mark.skipif(dy is None, reason="dataframely needs to be installed")
+@pytest.mark.skipif(dy.Column is None, reason="dataframely needs to be installed")
 def test_filter_with_filter_without_rule_violation():
     @materialize(input_type=pl.LazyFrame)
     def assertions(out, failure, failure_counts: dict[str, int]):
@@ -234,7 +226,7 @@ def test_filter_with_filter_without_rule_violation():
 
 
 @pytest.mark.skipif(cs.Collection is object, reason="ColSpec needs to be installed")
-@pytest.mark.skipif(dy is None, reason="dataframely needs to be installed")
+@pytest.mark.skipif(dy.Column is None, reason="dataframely needs to be installed")
 def test_filter_with_filter_with_rule_violation():
     @materialize(input_type=pl.LazyFrame)
     def assertions(out, failure, failure_counts: dict[str, int]):
@@ -268,13 +260,29 @@ def test_filter_with_filter_with_rule_violation():
     flow.run()
 
 
+@skip_instances("snowflake")
 @pytest.mark.skipif(cs.Collection is object, reason="ColSpec needs to be installed")
-@pytest.mark.skipif(dy is None, reason="dataframely needs to be installed")
+@pytest.mark.skipif(dy.Column is None, reason="dataframely needs to be installed")
 @pytest.mark.parametrize(
     "with_filter, with_violation, validate_get_data",
     [(a, b, c) for a in [False, True] for b in [False, True] for c in [False, True]],
 )
 def test_annotations(with_filter: bool, with_violation: bool, validate_get_data: bool):
+    do_test_annotations(with_filter, with_violation, validate_get_data)
+
+
+@with_instances("snowflake")
+@pytest.mark.skipif(cs.Collection is object, reason="ColSpec needs to be installed")
+@pytest.mark.skipif(dy.Column is None, reason="dataframely needs to be installed")
+@pytest.mark.parametrize(
+    "with_filter, with_violation, validate_get_data",
+    [(a, b, c) for a in [False] for b in [False, True] for c in [False]],
+)
+def test_annotations_snowflake(with_filter: bool, with_violation: bool, validate_get_data: bool):
+    do_test_annotations(with_filter, with_violation, validate_get_data)
+
+
+def do_test_annotations(with_filter: bool, with_violation: bool, validate_get_data: bool):
     if validate_get_data:
 
         @materialize(nout=2)
@@ -329,7 +337,7 @@ def test_annotations(with_filter: bool, with_violation: bool, validate_get_data:
 
     if with_violation and validate_get_data:
         # Validation at end of get_anno_data task fails
-        with pytest.raises(
+        with swallowing_raises(
             HookCheckException,
             match="failed validation with MyFirstColSpec; Failure counts: "
             "{'b|min': 1, 'c|nullability': 1, 'c|dtype': 1};"
@@ -340,7 +348,7 @@ def test_annotations(with_filter: bool, with_violation: bool, validate_get_data:
     elif with_violation and not validate_get_data and with_filter:
         # Due to the enum failure the dematerialization hook for consumer
         # task fails with ValueError (triggers RuntimeError in Flow)
-        with pytest.raises(
+        with swallowing_raises(
             RuntimeError,
             match="Failed to retrieve table '<Table 'get_anno_data",
         ):
@@ -350,13 +358,29 @@ def test_annotations(with_filter: bool, with_violation: bool, validate_get_data:
         assert ret.successful
 
 
+@skip_instances("snowflake")
 @pytest.mark.skipif(cs.Collection is object, reason="ColSpec needs to be installed")
-@pytest.mark.skipif(dy is None, reason="dataframely needs to be installed")
+@pytest.mark.skipif(dy.Column is None, reason="dataframely needs to be installed")
 @pytest.mark.parametrize(
     "with_filter, with_violation, validate_get_data",
     [(a, b, c) for a in [False, True] for b in [False, True] for c in [False, True]],
 )
 def test_annotations_not_fail_fast(with_filter: bool, with_violation: bool, validate_get_data: bool):
+    do_test_annotations_not_fail_fast(with_filter, with_violation, validate_get_data)
+
+
+@with_instances("snowflake")
+@pytest.mark.skipif(cs.Collection is object, reason="ColSpec needs to be installed")
+@pytest.mark.skipif(dy.Column is None, reason="dataframely needs to be installed")
+@pytest.mark.parametrize(
+    "with_filter, with_violation, validate_get_data",
+    [(a, b, c) for a in [True] for b in [False, True] for c in [False]],
+)
+def test_annotations_not_fail_fast_snowflake(with_filter: bool, with_violation: bool, validate_get_data: bool):
+    do_test_annotations_not_fail_fast(with_filter, with_violation, validate_get_data)
+
+
+def do_test_annotations_not_fail_fast(with_filter: bool, with_violation: bool, validate_get_data: bool):
     if validate_get_data:
 
         @materialize(nout=2)
@@ -403,7 +427,7 @@ def test_annotations_not_fail_fast(with_filter: bool, with_violation: bool, vali
         with Stage("s02"):
             consumer2(first, second)
 
-    with ConfigContext.get().evolve(fail_fast=False):
+    with ConfigContext.get().evolve(fail_fast=False, swallow_exceptions=True):
         result = flow.run(cache_validation_mode=CacheValidationMode.FORCE_CACHE_INVALID)
     if with_violation:
         assert not result.successful
@@ -411,13 +435,29 @@ def test_annotations_not_fail_fast(with_filter: bool, with_violation: bool, vali
         assert result.successful
 
 
+@skip_instances("snowflake")
 @pytest.mark.skipif(cs.Collection is object, reason="ColSpec needs to be installed")
-@pytest.mark.skipif(dy is None, reason="dataframely needs to be installed")
+@pytest.mark.skipif(dy.Column is None, reason="dataframely needs to be installed")
 @pytest.mark.parametrize(
     "with_filter, with_violation, validate_get_data",
     [(a, b, c) for a in [False, True] for b in [False, True] for c in [False, True]],
 )
 def test_annotations_fault_tolerant(with_filter: bool, with_violation: bool, validate_get_data: bool):
+    do_test_annotations_fault_tolerant(with_filter, with_violation, validate_get_data)
+
+
+@with_instances("snowflake")
+@pytest.mark.skipif(cs.Collection is object, reason="ColSpec needs to be installed")
+@pytest.mark.skipif(dy.Column is None, reason="dataframely needs to be installed")
+@pytest.mark.parametrize(
+    "with_filter, with_violation, validate_get_data",
+    [(a, b, c) for a in [False] for b in [False, True] for c in [True]],
+)
+def test_annotations_fault_tolerant_snowflake(with_filter: bool, with_violation: bool, validate_get_data: bool):
+    do_test_annotations_fault_tolerant(with_filter, with_violation, validate_get_data)
+
+
+def do_test_annotations_fault_tolerant(with_filter: bool, with_violation: bool, validate_get_data: bool):
     if validate_get_data:
 
         @materialize(nout=2)
@@ -476,9 +516,13 @@ def test_annotations_fault_tolerant(with_filter: bool, with_violation: bool, val
         with Stage("s02"):
             consumer2(first, second)
 
+    # result = flow.run(cache_validation_mode=CacheValidationMode.FORCE_CACHE_INVALID)
+    logger = structlog.get_logger(__name__ + ".test_annotations_fault_tolerant")
+    logger.info("Capturing logs...")
     with structlog.testing.capture_logs() as logs:
         with ConfigContext.get().evolve(table_hook_args=dict(polars=dict(fault_tolerant_annotation_action=True))):
             result = flow.run(cache_validation_mode=CacheValidationMode.FORCE_CACHE_INVALID)
+    logger.info("Continue logging")
     assert result.successful
     failures = [c for c in logs if c["event"] == "Failed to apply materialize annotation for table"]
     if with_violation and validate_get_data:
@@ -486,8 +530,9 @@ def test_annotations_fault_tolerant(with_filter: bool, with_violation: bool, val
         assert all("failed validation with My" in failure["exception"] for failure in failures)
 
 
+@skip_instances("snowflake")
 @pytest.mark.skipif(cs.Collection is object, reason="ColSpec needs to be installed")
-@pytest.mark.skipif(dy is None, reason="dataframely needs to be installed")
+@pytest.mark.skipif(dy.Column is None, reason="dataframely needs to be installed")
 @pytest.mark.parametrize(
     "with_filter, with_violation, validate_get_data",
     [(a, b, c) for a in [False, True] for b in [False, True] for c in [False, True]],
@@ -521,7 +566,9 @@ def test_collections(with_filter: bool, with_violation: bool, validate_get_data:
 
         if with_violation:
             with pytest.raises(cs.exc.MemberValidationError, match="2 members failed validation"):
-                coll.validate_polars(cast=True)
+                with structlog.testing.capture_logs() as logs:
+                    coll.validate_polars(cast=True)
+            assert logs == [{"exc_info": True, "event": "Dataframely validation failed", "log_level": "error"}]
         else:
             if with_filter:
                 # it is not really without violation
@@ -567,8 +614,9 @@ def test_collections(with_filter: bool, with_violation: bool, validate_get_data:
     assert ret.successful
 
 
+@skip_instances("snowflake")
 @pytest.mark.skipif(cs.Collection is object, reason="ColSpec needs to be installed")
-@pytest.mark.skipif(dy is None, reason="dataframely needs to be installed")
+@pytest.mark.skipif(dy.Column is None, reason="dataframely needs to be installed")
 def test_type_mapping():
     @materialize(nout=2)
     def get_anno_data() -> tuple[MyFirstColSpec, MySecondColSpec]:
@@ -586,3 +634,25 @@ def test_type_mapping():
             consumer(first, second)
 
     flow.run(cache_validation_mode=CacheValidationMode.FORCE_CACHE_INVALID)
+
+
+@with_instances("snowflake")
+@pytest.mark.skipif(cs.Collection is object, reason="ColSpec needs to be installed")
+@pytest.mark.skipif(dy.Column is None, reason="dataframely needs to be installed")
+def test_type_mapping_snowflake():
+    @materialize(nout=2)
+    def get_anno_data() -> tuple[MyFirstColSpec, MySecondColSpec]:
+        return data_with_filter_without_rule_violation()
+
+    @materialize(input_type=sa.Table)
+    def consumer(first: MyFirstColSpec, second: MySecondColSpec):
+        assert isinstance(first.c.b.type, sa.BigInteger)  # there is no SmallInteger in Snowflake
+        assert isinstance(second.c.b.type, sa.BigInteger)
+        assert not isinstance(second.c.b.type, sa.SmallInteger)
+
+    with Flow() as flow:
+        with Stage("s01"):
+            first, second = get_anno_data()
+            consumer(first, second)
+
+    flow.run(cache_validation_mode=CacheValidationMode.FORCE_CACHE_INVALID)
